"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var native_renderer_1 = require("./native-renderer");
var paperclip_utils_1 = require("paperclip-utils");
exports.patchNativeNode = function (mount, mutations, factory, protocol) {
    for (var _i = 0, mutations_1 = mutations; _i < mutations_1.length; _i++) {
        var mutation = mutations_1[_i];
        var target = getTarget(mount, mutation);
        var action = mutation.action;
        switch (action.kind) {
            case paperclip_utils_1.ActionKind.DeleteChild: {
                var child = target.childNodes[action.index];
                target.removeChild(child);
                break;
            }
            case paperclip_utils_1.ActionKind.InsertChild: {
                var newChild = native_renderer_1.createNativeNode(action.child, factory, protocol);
                if (action.index >= target.childNodes.length) {
                    target.appendChild(newChild);
                }
                else {
                    target.insertBefore(newChild, target.childNodes[action.index]);
                }
                break;
            }
            case paperclip_utils_1.ActionKind.ReplaceNode: {
                var parent_1 = target.parentNode;
                parent_1.insertBefore(native_renderer_1.createNativeNode(action.replacement, factory, protocol), target);
                target.remove();
                break;
            }
            case paperclip_utils_1.ActionKind.RemoveAttribute: {
                var element = target;
                element.removeAttribute(action.name);
                break;
            }
            case paperclip_utils_1.ActionKind.SetAttribute: {
                var element = target;
                element.setAttribute(action.name, action.value || "");
                break;
            }
            case paperclip_utils_1.ActionKind.SetText: {
                var text = target;
                text.nodeValue = action.value;
                break;
            }
        }
    }
};
var getTarget = function (mount, mutation) {
    return mutation.nodePath.reduce(function (current, i) { return current.childNodes[i]; }, mount);
};
