"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var native_renderer_1 = require("./native-renderer");
var paperclip_utils_1 = require("paperclip-utils");
var events_1 = require("events");
var utils_1 = require("./utils");
var virt_patcher_1 = require("./virt-patcher");
var dom_patcher_1 = require("./dom-patcher");
var RenderEventTypes;
(function (RenderEventTypes) {
    RenderEventTypes["META_CLICK"] = "META_CLICK";
})(RenderEventTypes || (RenderEventTypes = {}));
var Renderer = /** @class */ (function () {
    function Renderer(protocol, _domFactory) {
        var _this = this;
        if (_domFactory === void 0) { _domFactory = document; }
        this.protocol = protocol;
        this._domFactory = _domFactory;
        this.onMetaClick = function (listener) {
            _this._em.addListener(RenderEventTypes.META_CLICK, listener);
        };
        this.handleEngineEvent = function (event) {
            switch (event.kind) {
                case paperclip_utils_1.EngineEventKind.Evaluated: {
                    while (_this._stage.childNodes.length) {
                        _this._stage.removeChild(_this._stage.childNodes[0]);
                    }
                    _this._virtualRootNode = event.node;
                    var node = native_renderer_1.createNativeNode(event.node, _this._domFactory, _this.protocol);
                    _this._stage.appendChild(node);
                    break;
                }
                case paperclip_utils_1.EngineEventKind.Diffed: {
                    dom_patcher_1.patchNativeNode(_this._stage, event.mutations, _this._domFactory, _this.protocol);
                    _this._virtualRootNode = virt_patcher_1.patchVirtNode(_this._virtualRootNode, event.mutations);
                    break;
                }
            }
        };
        this._onStageMouseDown = function (event) {
            event.preventDefault();
            event.stopImmediatePropagation();
            var element = event.target;
            if (element.nodeType !== 1 || !event.metaKey)
                return;
            var nodePath = native_renderer_1.getNativeNodePath(_this.mount, element);
            var virtNode = virt_patcher_1.getVirtTarget(_this._virtualRootNode, nodePath);
            if (!virtNode)
                return;
            _this._em.emit(RenderEventTypes.META_CLICK, virtNode);
        };
        this._onStageMouseOver = function (event) {
            var element = event.target;
            var elementWindow = element.ownerDocument.defaultView;
            if (element.nodeType !== 1 || !event.metaKey)
                return;
            var rect = element.getBoundingClientRect();
            Object.assign(_this._hoverOverlay.style, {
                display: "block",
                left: elementWindow.document.body.scrollLeft + rect.left + "px",
                top: elementWindow.document.body.scrollTop + rect.top + "px",
                width: rect.width + "px",
                height: rect.height + "px"
            });
        };
        this._onStageMouseOut = function (event) {
            var element = event.target;
            if (element.nodeType !== 1)
                return;
            Object.assign(_this._hoverOverlay.style, {
                display: "none"
            });
        };
        this._em = new events_1.EventEmitter();
        this._hoverOverlay = _domFactory.createElement("div");
        Object.assign(this._hoverOverlay.style, {
            position: "absolute",
            zIndex: 1024,
            display: "none",
            background: "rgba(124, 154, 236, 0.5)",
            width: "100px",
            height: "100px",
            pointerEvents: "none",
            top: "0px",
            left: "0px"
        });
        this._stage = this._domFactory.createElement("div");
        this.mount = this._domFactory.createElement("div");
        this.mount.appendChild(this._stage);
        this.mount.appendChild(this._hoverOverlay);
        this._stage.addEventListener("mousedown", this._onStageMouseDown, true);
        this._stage.addEventListener("mouseup", utils_1.preventDefault, true);
        this._stage.addEventListener("mouseover", this._onStageMouseOver);
        this._stage.addEventListener("mouseout", this._onStageMouseOut);
    }
    return Renderer;
}());
exports.Renderer = Renderer;
