"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var paperclip_1 = require("paperclip");
var renderer_1 = require("../renderer");
var utils_1 = require("./utils");
describe(__filename + "#", function () {
    [
        ["Hello World", "hello test"],
        ["<span></span>", "text"],
        [
            "\n        <part id=\"something\">\n        </part>\n        \n        <span></span>\n        <something />\n        <span>\n          aa\n          <div />\n        </span>\n      ",
            "\n        <part id=\"something\">\n          <span />\n        </part>\n        \n        <span></span>\n        <something />\n        <span>\n          aa\n          <div />\n        </span>\n      "
        ],
        [
            "\n        <span class=\"block\">\n        </span>\n      ",
            "\n      <span class=\"block2\">\n      </span>\n      ",
            "\n      <span class=\"block2\" id=\"test\">\n      </span>\n      ",
            "\n      <span id=\"test\">\n      </span>\n      "
        ]
    ].forEach(function (_a) {
        var initial = _a[0], changes = _a.slice(1);
        console.log(changes);
        it("can render " + initial + " -> " + changes.join("->"), function () {
            var engine = createMockEngine(initial);
            var renderer = new renderer_1.Renderer("file://", utils_1.mockDOMFactory);
            engine.onEvent(renderer.handleEngineEvent);
            engine.load(DUMMY_FILE_URI);
            for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
                var change = changes_1[_i];
                engine.updateVirtualFileContent(DUMMY_FILE_URI, change);
                var renderer2 = new renderer_1.Renderer("file://", utils_1.mockDOMFactory);
                var engine2 = createMockEngine(change);
                engine2.onEvent(renderer2.handleEngineEvent);
                engine2.load(DUMMY_FILE_URI);
                chai_1.expect(getRendererHTML(renderer)).to.eql(getRendererHTML(renderer2));
            }
        });
    });
});
var DUMMY_FILE_URI = "file:///dummy-file-name.pc";
var cleanWS = function (html) { return html.replace(/[\r\n\t\s]+/g, " ").trim(); };
var createMockEngine = function (source) {
    return new paperclip_1.Engine({
        io: {
            readFile: function (uri) {
                return source;
            },
            resolveFile: function (uri) { return uri; },
            fileExists: function () { return true; }
        }
    });
};
var getRendererHTML = function (renderer) {
    return cleanWS(renderer.mount.childNodes[0].innerHTML);
};
